local interfaceMngr = nil;

local Types = {}

local bulletinBoardPostsForm = {};
local bulletinBoardAddPostForm = {};

function Init()
	LoadTypes();
	interfaceMngr = GetInterfaceManager();

	bulletinBoardPostsForm.Form = interfaceMngr:CreateForm("Bulletin Board", "Script");
	
	local ribbonPage = bulletinBoardPostsForm.Form:CreateRibbonPage("Bulletin Board");
	ribbonPage:CreateButton("Refresh", GetClientImage("Refresh32"), "RefreshBB", "Bulletin Board");
	ribbonPage:CreateButton("Add Post", GetClientImage("Add32"), "AddPost", "Bulletin Board");
	
	bulletinBoardPostsForm.PostsGrid = bulletinBoardPostsForm.Form:CreateGrid("PostsGrid", "PostsGrid");
	bulletinBoardPostsForm.PostsGrid.TextVisible = false;
	
	bulletinBoardPostsForm.PostBodyMemoEdit = bulletinBoardPostsForm.Form:CreateMemoEdit("Post Text", "Post Text");
	bulletinBoardPostsForm.PostBodyMemoEdit.TextVisible = false;
	bulletinBoardPostsForm.PostBodyMemoEdit.Editor.Properties.ReadOnly = true;
	
	-- MainView will come back as a GridView rather than a BaseView since that is the lowest class in its inheritance tree.
	bulletinBoardPostsForm.PostsGrid.GridControl.MainView.FocusedRowChanged:Add(postsGridView_SelectedPostChanged);
	
	RefreshBB();
	
	bulletinBoardPostsForm.Form:LoadLayout("layout.xml");
	
	bulletinBoardPostsForm.Form:Show();
end

function GetConnection()
	return CreateDatabaseConnection(GetSetting("DatabaseConnectionPath"));
end

function RefreshBB()
	local connection = GetConnection();
	connection.QueryString = "SELECT DatePosted, Subject, Author, Location, Symbol, Body FROM Posts ORDER BY DatePosted DESC";
	
	connection:Connect();	
	local postsTable = connection:Execute();
	connection:Disconnect();
		
	bulletinBoardPostsForm.PostsGrid.PrimaryTable = postsTable;
	bulletinBoardPostsForm.PostsGrid.GridControl.MainView.OptionsBehavior.Editable = false;
	bulletinBoardPostsForm.PostsGrid.GridControl.MainView.Columns:get_Item("Body").Visible = false;
	bulletinBoardPostsForm.PostsGrid.GridControl.MainView.Columns:get_Item("DatePosted").Caption = "Date";
	bulletinBoardPostsForm.PostsGrid.GridControl.MainView.Columns:get_Item("DatePosted").Width = 130;
	bulletinBoardPostsForm.PostsGrid.GridControl.MainView.Columns:get_Item("DatePosted").DisplayFormat.FormatString = "g"
	bulletinBoardPostsForm.PostsGrid.GridControl.MainView.Columns:get_Item("Subject").Width = 300;
	bulletinBoardPostsForm.PostsGrid.GridControl.MainView.Columns:get_Item("Location").Width = 200;
	
end

function LoadTypes()
	luanet.load_assembly("System.Data");
	luanet.load_assembly("System.Data.OleDb");
	Types["OleDbType"] = luanet.import_type("System.Data.OleDb.OleDbType");
end

function postsGridView_SelectedPostChanged(sender, args)
	if bulletinBoardPostsForm.PostsGrid.GridControl.MainView:IsDataRow(args.FocusedRowHandle) then
		UpdatePostText();
	end
end

function UpdatePostText()
	bulletinBoardPostsForm.PostBodyMemoEdit.Value = bulletinBoardPostsForm.PostsGrid.GridControl.MainView:GetFocusedRowCellValue("Body");
end

function AddPost()
	if bulletinBoardAddPostForm.Form ~= nil then
		return;
	end
	bulletinBoardAddPostForm.Form = interfaceMngr:CreateForm("Add Post", "Script");
	
	bulletinBoardAddPostForm.SubjectTextEdit = bulletinBoardAddPostForm.Form:CreateTextEdit("Subject", "Subject");
	bulletinBoardAddPostForm.AuthorTextEdit = bulletinBoardAddPostForm.Form:CreateTextEdit("Author", "Author");
	bulletinBoardAddPostForm.LocationTextEdit = bulletinBoardAddPostForm.Form:CreateTextEdit("Location", "Location");
	bulletinBoardAddPostForm.SymbolTextEdit = bulletinBoardAddPostForm.Form:CreateTextEdit("Symbol", "Symbol");
	bulletinBoardAddPostForm.BodyMemoEdit = bulletinBoardAddPostForm.Form:CreateMemoEdit("Post", "Post");
	
	bulletinBoardAddPostForm.LocationTextEdit.Value = GetSetting("Location");
	bulletinBoardAddPostForm.SymbolTextEdit.Value = GetSetting("Symbol");
	
	local ribbonPage = bulletinBoardAddPostForm.Form:CreateRibbonPage("New Post");
	ribbonPage:CreateButton("Save", GetClientImage("Save32"), "SaveNewPost", "New Post");
	ribbonPage:CreateButton("Cancel", GetClientImage("Cancel32"), "CancelNewPost", "New Post");
	
	bulletinBoardAddPostForm.Form:Show();
end

function SaveNewPost()
	local connection = GetConnection();
	-- Technically OleDb parameters are positional, rather than named.  
	-- The addon connection system requires names anyway to help enforce readability standards and minimize the addon side differences between OleDb and SQL connections.
	connection.QueryString = "INSERT INTO Posts (Subject, Body, Author, Location, Symbol, DatePosted) VALUES (@Subject, @Body, @Author, @Location, @Symbol, NOW())";
	connection:AddParameter("@Subject", bulletinBoardAddPostForm.SubjectTextEdit.Value, Types["OleDbType"].VarChar);
	connection:AddParameter("@Body", bulletinBoardAddPostForm.BodyMemoEdit.Value, Types["OleDbType"].VarChar);
	connection:AddParameter("@Author", bulletinBoardAddPostForm.AuthorTextEdit.Value, Types["OleDbType"].LongVarChar);	
	connection:AddParameter("@Location", bulletinBoardAddPostForm.LocationTextEdit.Value, Types["OleDbType"].LongVarChar);	
	connection:AddParameter("@Symbol", bulletinBoardAddPostForm.SymbolTextEdit.Value, Types["OleDbType"].LongVarChar);	
	
	connection:Connect();
	connection:ExecuteNonQuery();
	connection:Disconnect();
	
	RefreshBB();
	
	UpdatePostText();
	
	CloseAddPostForm()
end

function CancelNewPost()
	CloseAddPostForm()
end

function CloseAddPostForm()
	bulletinBoardAddPostForm.Form:Close();
	bulletinBoardAddPostForm.Form = nil;
end
